from typing import Optional
from .base import ApiBase, ApiId


class ExperimentApi(ApiBase):
    def send(
        self,
        *,
        experiment: str,
        user_id: Optional[str] = None,
        anonymous_id: Optional[str] = None,
        variant_name: Optional[str] = None,
        timestamp: Optional[str] = None,
        **extras,
    ):
        payload = self.prepare_payload({
            "user_id": user_id,
            "anonymous_id": anonymous_id,
            "variant_name": variant_name,
            "timestamp": timestamp,
        }, extras, ApiId.EXPERIMENT_SEND)

        path = f"v1/experiments/{experiment}/events"
        return self._api_call(path, payload)



from typing import Dict, List, Union
from .base import ApiBase, ApiId


class BulkApi(ApiBase):
    def bulk(self, *, requests: List[Dict[str, Union[str, dict]]], **extras):
        payload = self.prepare_payload({
            "requests": requests,
        }, extras, ApiId.BULK)
        return self._api_call('v1/bulk', payload)



from typing import Any, Dict, List
from .base import ApiBase, ApiId


class InteractionApi(ApiBase):
    def upload(self, *, data: List[Dict[str, Any]], **extras):
        payload = self.prepare_payload({
            "data": data,
        }, extras, ApiId.INTERACTION_UPLOAD)
        return self._api_call('v1/interactions', payload)


    def delete(self, *, user_ids: List[str], **extras):
        payload = self.prepare_payload({
            "user_ids": user_ids,
        }, extras, ApiId.INTERACTION_DELETE)
        return self._api_call('v1/interactions', payload, method='DELETE')



import requests

from .api.bulk import BulkApi
from .api.experiment import ExperimentApi
from .api.interaction import InteractionApi
from .api.product import ProductApi
from .api.qa import QAApi
from .api.recommendation import RecommendationApi
from .api.search import SearchApi
from .api.user import UserApi
from .version import SDK_VERSION


class ApiException(Exception):
    def __init__(self, message, status_code: int, response: dict):
        super().__init__(message)
        self.status_code = status_code
        self.response = response


class ApiClient:
    VERSION = SDK_VERSION

    def __init__(
        self,
        api_key: str,
        host: str = 'https://api.askmiso.com',
        raise_on_error: bool = False,
    ):
        session = requests.Session()

        session.params = {
            'api_key': api_key,
        }

        session.headers.update({
            'User-Agent': f'miso-sdk-python/{self.VERSION}',
        })

        def api_call(
            path: str,
            payload: dict,
            method: str = 'POST'
        ):
            url = f'{host}/{path}'

            resp = session.request(method, url, json=payload)
            data = resp.json()

            status = resp.status_code
            if raise_on_error and status >= 400:
                msg = f"API requset failed, status = {status}, message = `{data.get('message')}`"
                raise ApiException(msg, resp.status_code, data)
            return resp.json()

        self.interaction = InteractionApi(api_call)
        self.product = ProductApi(api_call)
        self.user = UserApi(api_call)
        self.search = SearchApi(api_call)
        self.recommendation = RecommendationApi(api_call)
        self.qa = QAApi(api_call)
        self.bulk = BulkApi(api_call)
        self.experiment = ExperimentApi(api_call)



from typing import Dict, List, Optional, Union
from .base import ApiBase, ApiId


class QAApi(ApiBase):
    def question_answering(
        self,
        *,
        q: str,
        min_probability: float,
        rows: Optional[int] = None,
        fl: Optional[List[str]] = None,
        fq: Optional[str] = None,
        boost_fq: Optional[str] = None,
        **extras,
    ):
        payload = self.prepare_payload({
            "q": q,
            "min_probability": min_probability,
            "rows": rows,
            "fl": fl,
            "fq": fq,
            "boost_fq": boost_fq,
        }, extras, ApiId.QA_ANSWER)
        return self._api_call('v1/qa/question_answering', payload)


    def upload_question_bank(
        self,
        *,
        data: List[Dict[str, Union[str, float]]],
        **extras,
    ):
        payload = self.prepare_payload({
            "data": data,
        }, extras, ApiId.QA_UPLOAD)
        return self._api_call('v1/qa/questions', payload)


    def question_autocomplete(
        self,
        *,
        q: str,
        rows: Optional[int] = None,
        **extras,
    ):
        payload = self.prepare_payload({
            "q": q,
            "rows": rows,
        }, extras, ApiId.QA_AUTOCOMPLETE)
        return self._api_call('v1/qa/question_autocomplete', payload)



from typing import List, Optional
from .base import ApiBase, ApiId


class SearchApi(ApiBase):
    def search(
        self,
        *,
        q: str,
        user_id: Optional[str] = None,
        anonymous_id: Optional[str] = None,
        fl: Optional[List[str]] = None,
        fq: Optional[str] = None,
        boost_fq: Optional[str] = None,
        start: Optional[int] = None,
        rows: Optional[int] = None,
        **extras,
    ):

        payload = self.prepare_payload({
            'q': q,
            'user_id': user_id,
            'anonymous_id': anonymous_id,
            'fl': fl,
            'fq': fq,
            'boost_fq': boost_fq,
            'start': start,
            'rows': rows,
        }, extras, ApiId.SEARCH)
        return self._api_call('v1/search/search', payload)


    def autocomplete(
        self,
        *,
        q: str,
        user_id: Optional[str] = None,
        anonymous_id: Optional[str] = None,
        completion_fields: Optional[List[str]] = None,
        fl: Optional[List[str]] = None,
        start: Optional[int] = None,
        rows: Optional[int] = None,
        **extras,
    ):
        payload = self.prepare_payload({
            'q': q,
            'user_id': user_id,
            'anonymous_id': anonymous_id,
            'completion_fields': completion_fields,
            'fl': fl,
            'start': start,
            'rows': rows,
        }, extras, ApiId.AUTOCOMPLETE)

        return self._api_call('v1/search/autocomplete', payload)


    def multi_get(
        self,
        *,
        product_ids: List[str],
        fl: Optional[List[str]] = None,
        **extras,
    ):
        payload = self.prepare_payload({
            'product_ids': product_ids,
            'fl': fl,
        }, extras, ApiId.MGET)
        return self._api_call('v1/search/mget', payload)



from typing import List, Optional
from .base import ApiBase, ApiId


class RecommendationApi(ApiBase):
    def user_to_products(
        self,
        *,
        user_id: Optional[str] = None,
        anonymous_id: Optional[str] = None,
        rows: Optional[int] = None,
        type: Optional[str] = None,
        fl: Optional[List[str]] = None,
        fq: Optional[str] = None,
        boost_fq: Optional[str] = None,
        pagination_id: Optional[str] = None,
        start: Optional[int] = None,
        **extras
    ):
        payload = self.prepare_payload({
            "user_id": user_id,
            "anonymous_id": anonymous_id,
            "rows": rows,
            "type": type,
            "fl": fl,
            "fq": fq,
            "boost_fq": boost_fq,
            "pagination_id": pagination_id,
            "start": start,
        }, extras, ApiId.REC_U2I)
        return self._api_call('v1/recommendation/user_to_products', payload)


    def user_to_categories(
        self,
        *,
        user_id: Optional[str] = None,
        anonymous_id: Optional[str] = None,
        rows: Optional[int] = None,
        type: Optional[str] = None,
        fl: Optional[List[str]] = None,
        products_per_category: Optional[int] = None,
        root_category: Optional[List[str]] = None,
        fq: Optional[str] = None,
        boost_fq: Optional[str] = None,
        **extras
    ):
        payload = self.prepare_payload({
            "user_id": user_id,
            "anonymous_id": anonymous_id,
            "rows": rows,
            "type": type,
            "fl": fl,
            "products_per_category": products_per_category,
            "root_category": root_category,
            "fq": fq,
            "boost_fq": boost_fq,
        }, extras, ApiId.REC_U2C)
        return self._api_call('v1/recommendation/user_to_categories', payload)


    def user_to_attributes(
        self,
        *,
        field: str,
        boost_attributes: Optional[List[str]] = None,
        user_id: Optional[str] = None,
        anonymous_id: Optional[str] = None,
        rows: Optional[int] = None,
        type: Optional[str] = None,
        fl: Optional[List[str]] = None,
        products_per_attribute: Optional[int] = None,
        fq: Optional[str] = None,
        boost_fq: Optional[str] = None,
        **extras
    ):
        payload = self.prepare_payload({
            "field": field,
            "boost_attributes": boost_attributes,
            "user_id": user_id,
            "anonymous_id": anonymous_id,
            "rows": rows,
            "type": type,
            "fl": fl,
            "products_per_attribute": products_per_attribute,
            "fq": fq,
            "boost_fq": boost_fq,
        }, extras, ApiId.REC_U2A)
        return self._api_call('v1/recommendation/user_to_attributes', payload)


    def user_to_trending(
        self,
        *,
        user_id: Optional[str] = None,
        anonymous_id: Optional[str] = None,
        rows: Optional[int] = None,
        type: Optional[str] = None,
        fl: Optional[List[str]] = None,
        fq: Optional[str] = None,
        boost_fq: Optional[str] = None,
        pagination_id: Optional[str] = None,
        start: Optional[int] = None,
        **extras
    ):
        payload = self.prepare_payload({
            "user_id": user_id,
            "anonymous_id": anonymous_id,
            "rows": rows,
            "type": type,
            "fl": fl,
            "fq": fq,
            "boost_fq": boost_fq,
            "pagination_id": pagination_id,
            "start": start,
        }, extras, ApiId.REC_U2T)
        return self._api_call('v1/recommendation/user_to_trending', payload)


    def product_to_products(
        self,
        *,
        user_id: Optional[str] = None,
        anonymous_id: Optional[str] = None,
        rows: Optional[int] = None,
        type: Optional[str] = None,
        fl: Optional[List[str]] = None,
        product_id: Optional[str] = None,
        product_ids: Optional[List[str]] = None,
        product_group_id: Optional[str] = None,
        product_group_ids: Optional[List[str]] = None,
        fq: Optional[str] = None,
        boost_fq: Optional[str] = None,
        pagination_id: Optional[str] = None,
        start: Optional[int] = None,
        **extras
    ):
        payload = self.prepare_payload({
            "user_id": user_id,
            "anonymous_id": anonymous_id,
            "rows": rows,
            "type": type,
            "fl": fl,
            "product_id": product_id,
            "product_ids": product_ids,
            "product_group_id": product_group_id,
            "product_group_ids": product_group_ids,
            "fq": fq,
            "boost_fq": boost_fq,
            "pagination_id": pagination_id,
            "start": start,
        }, extras, ApiId.REC_I2I)
        return self._api_call('v1/recommendation/product_to_products', payload)



import pytest

from miso.api.base import ApiBase, ApiId


def null_func():
    pass


@pytest.mark.parametrize('api_id, payload', [
    (ApiId.BULK, {"data":[
        {"api_name":"search/search", "body": {"user_id": "usr", "q": "star", "fq":"title:love"}},
    ]}),
    (ApiId.EXPERIMENT_SEND, {"experiment": "a", "user_id": "usr"}),
    (ApiId.INTERACTION_UPLOAD, {"data": [{"user_id":"usr","type":"search"}]}),
    (ApiId.INTERACTION_DELETE, {}),
    (ApiId.PRODUCT_UPLOAD, {"data":[{}]}),
    (ApiId.PRODUCT_READ, {}),
    (ApiId.PRODUCT_DELETE, {}),
    (ApiId.QA_ANSWER, {"q": "what is love", "min_probability": 0}),
    (ApiId.QA_ANSWER, {"q": "what is love", "min_probability": 0.3}),
    (ApiId.QA_UPLOAD, {}),
    (ApiId.QA_AUTOCOMPLETE, {}),
    (ApiId.REC_U2I, {"user_id": "usr",}),
    (ApiId.REC_U2C, {"user_id": "usr",}),
    (ApiId.REC_U2A, {"user_id": "usr",}),
    (ApiId.REC_U2T, {"user_id": "usr",}),
    (ApiId.REC_I2I, {"user_id": "usr", "product_ids":["1", "2", "3"]}),
    (ApiId.SEARCH, {"user_id": "usr", "q": "star"}),
    (ApiId.AUTOCOMPLETE, {"user_id": "usr", "q": "aut"}),
    (ApiId.MGET, {"product_ids":["1", "2", "3"]}),
    (ApiId.USER_UPLOAD, {"data":[{}]}),
    (ApiId.USER_READ, {}),
    (ApiId.USER_DELETE, {}),
])
def test_validation_pass(api_id, payload):
    api_base = ApiBase(null_func)
    api_base.validate_payload(api_id, payload)


@pytest.mark.parametrize('api_id, payload', [
    (ApiId.BULK, {"data":{"api_name":"search/search", "body": {"user_id": "usr", "q": "star", "fq":"title:love"}}}),
    (ApiId.BULK, {"data":['star', 'trek', 'war', 'lord']}),
    (ApiId.EXPERIMENT_SEND, {"experiment": "a", "user_id": 123}),
    (ApiId.INTERACTION_UPLOAD, {"data": [1,2,3]}),
    (ApiId.INTERACTION_UPLOAD, {"data": {"user_id": "usr", "type": "search"}}),
    (ApiId.PRODUCT_UPLOAD, {"data":['1','2','3']}),
    (ApiId.PRODUCT_UPLOAD, {"data":{"id":"prod1", "title": "good product"}}),
    (ApiId.PRODUCT_READ, {"product_id": 123}),
    (ApiId.QA_ANSWER, {"q": ["what is love"], "min_probability": 0}),
    (ApiId.REC_I2I, {"user_id": "usr", "product_id":["1", "2", "3"]}),
    (ApiId.REC_I2I, {"user_id": "usr", "product_ids": "1,2,3"}),
    (ApiId.MGET, {"product_id":["1", "2", "3"]}),
    (ApiId.USER_UPLOAD, {"data":{}}),
])
def test_validation_fail(api_id, payload):
    api_base = ApiBase(null_func)
    with pytest.raises(ValueError, match=r".* shoud be .*"):
        api_base.validate_payload(api_id, payload)


@pytest.mark.parametrize('api_id, payload', [
    (ApiId.REC_U2I, {}),
    (ApiId.REC_U2C, {}),
    (ApiId.REC_U2A, {}),
    (ApiId.REC_U2T, {}),
    (ApiId.REC_I2I, {"product_ids":["1", "2", "3"]}),
    (ApiId.SEARCH, {"q": "star"}),
    (ApiId.AUTOCOMPLETE, {"q": "aut"}),
])
def test_validation_need_user(api_id, payload):
    api_base = ApiBase(null_func)
    with pytest.raises(ValueError, match="Either user_id or anonymous_id need to be present"):
        api_base.validate_payload(api_id, payload)



from enum import auto, Enum
from typing import Any, Callable, Dict


class ApiId(Enum):
    BULK = auto()
    EXPERIMENT_SEND = auto()
    INTERACTION_UPLOAD = auto()
    INTERACTION_DELETE = auto()
    PRODUCT_UPLOAD = auto()
    PRODUCT_READ = auto()
    PRODUCT_DELETE = auto()
    QA_ANSWER = auto()
    QA_UPLOAD = auto()
    QA_AUTOCOMPLETE = auto()
    REC_U2I = auto()
    REC_U2C = auto()
    REC_U2A = auto()
    REC_U2T = auto()
    REC_I2I = auto()
    SEARCH = auto()
    AUTOCOMPLETE = auto()
    MGET = auto()
    USER_UPLOAD = auto()
    USER_READ = auto()
    USER_DELETE = auto()

Number = (int, float)

class ApiBase:
    def __init__(
        self,
        call_func: Callable,
    ):
        self._api_call = call_func


    def prepare_payload(
        self,
        inputs: Dict[str, Any],
        extras: Dict[str, Any],
        api_id: ApiId,
    ) -> Dict[str, Any]:
        none_keys = [k for k, v in inputs.items() if v is None]
        for key in none_keys:
            inputs.pop(key, None)

        inputs.update(extras)

        self.validate_payload(api_id, inputs)

        return inputs


    def validate_payload(self, api_id: ApiId, payload: Dict[str, Any]):
        errors = []

        check_user = {
            ApiId.REC_U2I,
            ApiId.REC_U2C,
            ApiId.REC_U2A,
            ApiId.REC_U2T,
            ApiId.REC_I2I,
            ApiId.SEARCH,
            ApiId.AUTOCOMPLETE,
        }
        if api_id in check_user:
            if 'user_id' not in payload and 'anonymous_id' not in payload:
                errors.append("Either user_id or anonymous_id need to be present")

        # basic type checking
        types = {
            str: {'user_id', 'anonymous_id', 'product_id', 'q', 'fq', 'boost_fq'},
            int: {'rows', 'start'},
            Number: {'min_probability', 'weight', 'duration', 'rating', 'revenue'},
        }
        list_str_fields = {'fl', 'user_ids', 'product_ids'}

        for field, val in payload.items():
            # built-in types
            for field_type, typed_fields in types.items():
                if field in typed_fields \
                   and not isinstance(val, field_type): # type: ignore[arg-type]
                    errors.append(f"`{field}` ({val}) shoud be {field_type}, not {type(val)}")
            # generic types
            if field in list_str_fields:
                if not isinstance(val, list) \
                   or not all(isinstance(v, str) for v in val):
                    errors.append(f"`{field}` ({val}) shoud be a list of string")

        # data list
        data_list_apis = {
            ApiId.BULK,
            ApiId.USER_UPLOAD,
            ApiId.PRODUCT_UPLOAD,
            ApiId.INTERACTION_UPLOAD,
        }
        if api_id in data_list_apis and 'data' in payload:
            data = payload['data']
            if not isinstance(data, list) \
               or not all(isinstance(v, dict) for v in data):
                errors.append(f"`data` ({data}) shoud be a list of dict")

        if len(errors) > 0:
            raise ValueError('; '.join(errors))



from typing import Any, Dict, List
from .base import ApiBase, ApiId


class ProductApi(ApiBase):
    def upload(self, *, data:List[Dict[str, Any]], **extras):
        payload = self.prepare_payload({
            "data": data,
        }, extras, ApiId.PRODUCT_UPLOAD)
        return self._api_call('v1/products', payload)


    def read(self, *, product_id: str, **extras):
        payload = self.prepare_payload({}, extras, ApiId.PRODUCT_READ)
        path = f'v1/products/{product_id}'
        return self._api_call(path, payload, method='GET')


    def delete(self, *, product_id: str, **extras):
        payload = self.prepare_payload({}, extras, ApiId.PRODUCT_DELETE)
        path = f'v1/products/{product_id}'
        return self._api_call(path, payload, method='DELETE')



import pytest

from miso.api.bulk import BulkApi
from miso.api.experiment import ExperimentApi
from miso.api.interaction import InteractionApi
from miso.api.product import ProductApi
from miso.api.qa import QAApi
from miso.api.recommendation import RecommendationApi
from miso.api.search import SearchApi
from miso.api.user import UserApi


def make_trap():
    store = dict()

    def trap(
        path: str,
        payload: dict,
        method: str = 'POST',
    ):
        store['path'] = path
        store['payload'] = payload
        store['method'] = method

    return store, trap


SAME = "same_payload_identifier"


params = [
    (
        BulkApi,
        'bulk',
        {"requests":[{"api_name":"search/search", "body": {"q":"star", "user_id": "usr"}}]},
        {"method": "POST", "path": "v1/bulk", "payload": SAME},
    ),
    (
        ExperimentApi,
        'send',
        {"experiment": "some_exp", "user_id": "usr"},
        {"method": "POST", "path": "v1/experiments/some_exp/events", "payload": {"user_id": "usr"}},
    ),
    (
        InteractionApi,
        'upload',
        {"data": [{}]},
        {"method": "POST", "path": "v1/interactions", "payload": SAME},
    ),
    (
        InteractionApi,
        'delete',
        {"user_ids": ["USR1", "USR2", "USR3"]},
        {"method": "DELETE", "path": "v1/interactions", "payload": SAME},
    ),
    (
        ProductApi,
        'upload',
        {"data": [{}]},
        {"method": "POST", "path": "v1/products", "payload": SAME},
    ),
    (
        ProductApi,
        'read',
        {"product_id": "ID0"},
        {"method": "GET", "path": f"v1/products/ID0", "payload": {}},
    ),
    (
        ProductApi,
        'delete',
        {"product_id": "ID0"},
        {"method": "DELETE", "path": f"v1/products/ID0", "payload": {}},
    ),
    (
        QAApi,
        'question_answering',
        {"q": "What is python", "min_probability": 0.3},
        {"method": "POST", "path": f"v1/qa/question_answering", "payload": SAME},
    ),
    (
        QAApi,
        'upload_question_bank',
        {"data": [{}]},
        {"method": "POST", "path": f"v1/qa/questions", "payload": SAME},
    ),
    (
        QAApi,
        'question_autocomplete',
        {"q": "what is wyvern"},
        {"method": "POST", "path": f"v1/qa/question_autocomplete", "payload": SAME},
    ),
    (
        RecommendationApi,
        'user_to_products',
        {"user_id": "usr", "rows": 10},
        {"method": "POST", "path": f"v1/recommendation/user_to_products", "payload": SAME},
    ),
    (
        RecommendationApi,
        'user_to_categories',
        {"user_id": "usr", "rows": 10},
        {"method": "POST", "path": f"v1/recommendation/user_to_categories", "payload": SAME},
    ),
    (
        RecommendationApi,
        'user_to_attributes',
        {"user_id": "usr", "rows": 10, "field": "title"},
        {"method": "POST", "path": f"v1/recommendation/user_to_attributes", "payload": SAME},
    ),
    (
        RecommendationApi,
        'user_to_trending',
        {"user_id": "usr", "rows": 10},
        {"method": "POST", "path": f"v1/recommendation/user_to_trending", "payload": SAME},
    ),
    (
        RecommendationApi,
        'product_to_products',
        {"user_id": "usr", "product_ids":["1701", "1701A"], "rows": 10},
        {"method": "POST", "path": f"v1/recommendation/product_to_products", "payload": SAME},
    ),
    (
        SearchApi,
        'search',
        {"q":"star", "user_id": "usr"},
        {"method": "POST", "path": "v1/search/search", "payload": SAME},
    ),
    (
        SearchApi,
        'autocomplete',
        {"q":"star", "user_id": "usr"},
        {"method": "POST", "path": "v1/search/autocomplete", "payload": SAME},
    ),
    (
        UserApi,
        'upload',
        {"data":[{}]},
        {"method": "POST", "path": "v1/users", "payload": SAME},
    ),
    (
        UserApi,
        'read',
        {"user_id": "ID0"},
        {"method": "GET", "path": f"v1/users/ID0", "payload": {}},
    ),
    (
        UserApi,
        'delete',
        {"user_id": "ID0"},
        {"method": "DELETE", "path": f"v1/users/ID0", "payload": {}},
    ),
]

param_ids = [f"{param[0].__name__}.{param[1]}" for param in params]

@pytest.mark.parametrize('api, func_name, input, expected', params, ids=param_ids)
def test_parameter_passing(api, func_name, input, expected):
    if expected['payload'] == SAME:
        expected['payload'] = {**input} # copy

    store, trap = make_trap()
    api = api(trap)
    func = getattr(api, func_name)

    func(**input)
    assert store == expected

    # Make sure extras works

    expected_extra = {**expected} # copy
    expected_extra['payload']['extra_param'] = 'extra_value'

    func(**input, extra_param="extra_value")
    assert store == expected_extra



from setuptools import setup, find_packages
import pathlib

from miso.version import SDK_VERSION

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="miso-sdk",
    version=SDK_VERSION,
    license='MIT',

    description="Enhance your site with high conversion magic with Miso's power.",
    long_description=long_description,
    long_description_content_type="text/markdown",

    url="https://github.com/MisoAI/miso-python-sdk",

    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],

    packages=find_packages(),

    python_requires=">=3.7, <4",
    install_requires=["requests"],

    project_urls={
        "Miso": "https://askmiso.com/",
        "Miso Document": "https://docs.askmiso.com/",
        "Miso API Document": "https://api.askmiso.com/",
    },
)



from typing import Any, Dict, List
from .base import ApiBase, ApiId


class UserApi(ApiBase):
    def upload(self, data: List[Dict[str, Any]], **extras):
        payload = self.prepare_payload({"data": data}, extras, ApiId.USER_UPLOAD)
        return self._api_call('v1/users', payload)


    def read(self, user_id: str, **extras):
        payload = self.prepare_payload({}, extras, ApiId.USER_READ)
        path = f'v1/users/{user_id}'
        return self._api_call(path, payload, method='GET')


    def delete(self, user_id: str, **extras):
        payload = self.prepare_payload({}, extras, ApiId.USER_DELETE)
        path = f'v1/users/{user_id}'
        return self._api_call(path, payload, method='DELETE')
